/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClientContainer.java 2981 2008-04-27 15:58:00Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.client;

import static org.ow2.easybeans.util.url.URLUtils.fileToURL2;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.LinkRef;
import javax.naming.NamingException;

import org.objectweb.carol.util.configuration.ConfigurationRepository;
import org.ow2.easybeans.client.xml.ApplicationClient;
import org.ow2.easybeans.client.xml.ApplicationClientLoader;
import org.ow2.easybeans.deployment.Deployment;
import org.ow2.easybeans.deployment.resolver.JNDIResolverHelper;
import org.ow2.easybeans.deployment.xml.struct.common.EJBRef;
import org.ow2.easybeans.deployment.xml.struct.common.InjectionTarget;
import org.ow2.easybeans.enhancer.EnhancerException;
import org.ow2.easybeans.loader.EasyBeansClassLoader;
import org.ow2.easybeans.naming.NamingManager;
import org.ow2.easybeans.resolver.ContainerJNDIResolver;
import org.ow2.easybeans.resolver.api.EZBContainerJNDIResolver;
import org.ow2.easybeans.resolver.api.EZBJNDIResolverException;
import org.ow2.easybeans.util.files.FileUtils;
import org.ow2.easybeans.util.files.FileUtilsException;
import org.ow2.easybeans.util.url.URLUtilsException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.archive.impl.ArchiveManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Defines the class use for the client container This class analyze the ear or
 * the jar client and launch the client.
 * @author Florent Benoit
 */
public final class ClientContainer {

    /**
     * Folder to create in tmp folder.
     */
    private static final String DEFAULT_FOLDER = "EasyBeans-Deployer";

    /**
     * Main class to use to launch the application client.
     */
    private String mainClass = null;

    /**
     * Temporary directory.
     */
    private String tmpDir = null;

    /**
     * Classpath for the application client.
     */
    private String classpath = null;

    /**
     * Arguments used by the client.
     */
    private String[] args = null;

    /**
     * Extra Arguments.
     */
    private List<String> appArgs = null;

    /**
     * URLs resolved in the case of the extension mechanism in the Ear case.
     */
    private URL[] extensionsURLs = null;

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(ClientContainer.class);


    /**
     * Reference to a JNDIResolver.
     */
    private EZBContainerJNDIResolver containerJNDIResolver = null;


    /**
     * Jar client to use (if many).
     */
    private String jarClient = null;

    /**
     * XML Struct representing application-client.xml file.
     */
    private ApplicationClient applicationClient = null;

    /**
     * Constructor for a Client container.
     * @param args the arguments of the instance of the client container
     */
    private ClientContainer(final String[] args) {
        this.args = args;
        this.appArgs = new ArrayList<String>();
    }

    /**
     * Main method of the Client container.
     * @param args the arguments of the client container
     */
    public static void main(final String[] args) {
        // Retrieve command line parameters
        ClientContainer cc = new ClientContainer(args);

        try {
            cc.start();
        } catch (InvocationTargetException ite) {
            Throwable t = ite.getTargetException();
            String message = t.getMessage();
            if (t instanceof Error) {
                System.err.println("There was the following error : " + message);
            } else if (t instanceof Exception) {
                System.err.println("There was the following exception : " + message);
            }
            t.printStackTrace(System.err);
        } catch (Exception e) {
            System.err.println("There was the following exception : " + e.getMessage());
            e.printStackTrace();
            System.exit(-1);
        }
    }

    /**
     * Start the client container.
     * @throws Exception if it fails
     */
    private void start() throws Exception {
        analyzeArgs();

        // Get the filename
        String userArg = null;
        String fileName = null;
        boolean fileMode = true;

        try {
            userArg = this.appArgs.get(0);
        } catch (IndexOutOfBoundsException ioobe) {
            usage();
            throw new ClientContainerException("You haven't specify a jar, an ear file or class name as argument."
                    + "See the Usage.");
        }

        String className = null;
        // Test if this is an ear or a jar file else it must be a class name
        if (!(userArg.toLowerCase().endsWith(".jar") || userArg.toLowerCase().endsWith(".ear"))) {
            className = userArg;
            fileMode = false;
        } else {
            fileMode = true;
            fileName = userArg;
        }

        // Build file and test if it exists
        File clientJarFile = null;
        if (fileMode) {
            File argFile = new File(fileName);

            // Unpack and analyze EAR file if it is an ear
            if (fileName.toLowerCase().endsWith(".ear")) {
                clientJarFile = extractAndAnalyzeEar(argFile);
            } else {
                // Client jar is the given file
                clientJarFile = argFile;
            }
        }

        // Carol initialisation (property)
        ConfigurationRepository.init();
        System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");
        System.setProperty(Context.URL_PKG_PREFIXES , "org.ow2.easybeans.naming.pkg");

        // Extract Main-Class to use in the jar from the manifest
        if (fileMode) {
            Manifest manifest = new JarFile(clientJarFile).getManifest();

            if (manifest == null) {
                throw new ClientContainerException("No manifest was found inside the file" + clientJarFile);
            }

            // Extract attributes
            Attributes attributes = manifest.getMainAttributes();

            if (attributes == null) {
                throw new ClientContainerException("No attributes were found in the manifest of the file '" + clientJarFile
                        + "'.");
            }
            this.mainClass = attributes.getValue(Attributes.Name.MAIN_CLASS);
        } else {
            this.mainClass = className;
        }

        // Invoke the client if there is no need of XML parsing
        if (!fileMode) {
            ClassLoader clientCL = new URLClassLoader(getUserClasspathUrls());
            Thread.currentThread().setContextClassLoader(clientCL);
            invokeClient();
            return;
        }

        if (this.mainClass == null || this.mainClass.length() == 0) {
            throw new ClientContainerException("No main class was found inside the Manifest  of the file '" + clientJarFile
                    + "'. This attribute is required to launch the application client.");
        }

        this.logger.info("Using Main-Class : {0}", this.mainClass);


        // Convert file to URL
        URL clientJarURL = null;

        try {
            clientJarURL = fileToURL2(clientJarFile);
        } catch (URLUtilsException uue) {
            throw new ClientContainerException("Error when building an URL with the file '" + clientJarFile + "'.", uue);
        }

        // Parse xml file (if any)
        this.applicationClient = ApplicationClientLoader.loadApplicationClient(clientJarFile);


        // Build the urls for the classloader
        URL[] urlsClient = null;

        // URLs for the classloader
        if (this.extensionsURLs != null) {
            // There were URLs with the extension mechanism in the EAR
            urlsClient = new URL[this.extensionsURLs.length + 1];

            for (int i = 0; i < this.extensionsURLs.length; i++) {
                urlsClient[i] = this.extensionsURLs[i];

                this.logger.debug("Adding " + this.extensionsURLs[i] + " to the urls of the client");
            }

            urlsClient[this.extensionsURLs.length] = clientJarURL;
        } else {
            this.logger.debug("Only one url for urls of client");

            // No extension or jar case.
            urlsClient = new URL[1];
            urlsClient[0] = clientJarURL;
        }

        // Build classloader
        this.logger.debug("Building classloader with urls {0}", Arrays.asList(urlsClient));
        EasyBeansClassLoader clientClassloader = new EasyBeansClassLoader(urlsClient,
                Thread.currentThread().getContextClassLoader());
        Thread.currentThread().setContextClassLoader(clientClassloader);


        // Build environment of the client.
        buildENC();

        // Start client
        invokeClient();

    }

    /**
     * Start the client on its main class with the thread class loader.
     * @throws ClassNotFoundException if class is not found
     * @throws NoSuchMethodException if method (main) is not found
     * @throws IllegalAccessException if access is illegal
     * @throws InvocationTargetException if invocation failed
     * @throws EnhancerException if enhancement fails.
     * @throws ClientContainerException if there is a failure.
     */
    private void invokeClient() throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException,
            InvocationTargetException, EnhancerException, ClientContainerException {
        ClassLoader clientClassloader = Thread.currentThread().getContextClassLoader();

        if (this.logger.isDebugEnabled()) {
            if (clientClassloader instanceof URLClassLoader) {
                URLClassLoader urlClassLoader = (URLClassLoader) clientClassloader;
                URL[] urls = urlClassLoader.getURLs();
                this.logger.debug("URLs of the classloader :");
                for (int u = 0; u < urls.length; u++) {
                    this.logger.debug("URL[" + u + "] = " + urls[u]);
                }
            }
        }



        // Need to enhance client class (for injected resources).
        List<String> classesToEnhance = new ArrayList<String>();
        classesToEnhance.add(this.mainClass.replace(".", "/"));

        // Add resolver in the map
        Map<String, Object> map = new HashMap<String, Object>();
        map.put(EZBContainerJNDIResolver.class.getName(), this.containerJNDIResolver);

        // Enhance client code
        ClientEnhancer.enhance(Thread.currentThread().getContextClassLoader(), classesToEnhance, map);

        // Invoke client
        // Launch the "class_to_run" by using our classloader.
        Class<?> clazz = clientClassloader.loadClass(this.mainClass);
        Class<?>[] argList = new Class[] {this.args.getClass()};

        // ejb-ref injection code
        if (this.applicationClient != null) {
            for (EJBRef ejbRef : this.applicationClient.getEJBRefs()) {
                List<InjectionTarget> injectionTargetList = ejbRef.getInjectionTargetList();
                if (injectionTargetList != null) {
                    for (InjectionTarget injectionTarget : injectionTargetList) {
                        // get target name
                        String targetName = injectionTarget.getTargetName();
                        try {
                            Field field = clazz.getDeclaredField(targetName);
                            boolean access = field.isAccessible();
                            try {
                                if (!access) {
                                    field.setAccessible(true);
                                }
                                //  ejb-link
                                String ejbLink = ejbRef.getEjbLink();
                                // interface
                                String itfName = ejbRef.getRemote();

                                // Gets the jndi name
                                String jndiName;
                                try {
                                    jndiName = this.containerJNDIResolver.getUniqueName(itfName, ejbLink);
                                } catch (EZBJNDIResolverException e) {
                                    throw new ClientContainerException("Cannot find the expected JNDI name", e);
                                }
                                if (jndiName != null) {
                                    Object value = null;
                                    try {
                                        value = new InitialContext().lookup(jndiName);
                                    } catch (NamingException e) {
                                        this.logger.error("Unable to lookup the name '" + jndiName + "'.", e);
                                    }
                                    if (value != null) {
                                        field.set(this.applicationClient, value);
                                    } else {
                                        this.logger.error("No value found in Initial Context for the name '" + jndiName + "'.");
                                    }
                                } else {
                                    this.logger.error("No JNDI name found for interface '" + itfName + "'.");
                                }
                            } finally {
                                if (!access) {
                                    field.setAccessible(false);
                                }
                            }

                        } catch (SecurityException e) {
                            this.logger.error("Cannot perform injection on the client", e);
                        } catch (NoSuchFieldException e) {
                            this.logger.error("Cannot perform injection on the client", e);
                        }
                    }
                }
            }

        }

        // First, call injection code
        Method injectedMeth = clazz.getMethod("injectedByEasyBeans", new Class[] {});
        try {
            injectedMeth.invoke(null, new Object[] {});
        } catch (Exception e) {
            this.logger.error("Cannot perform injection on the client", e);
        }

        // Then, call lifecycle method
        Method lifeCycleMeth = clazz.getMethod("easyBeansLifeCyclePostConstruct", new Class[] {});
        try {
            lifeCycleMeth.invoke(null, new Object[] {});
        } catch (Exception e) {
            this.logger.error("Cannot perform lifecycle init on the client", e);
        }


        Method meth = clazz.getMethod("main", argList);

        // Remove name of the file from arguments
        String[] newArgs = new String[this.appArgs.size() - 1];
        String txtArgs = "";

        for (int i = 0; i < newArgs.length; i++) {
            newArgs[i] = this.appArgs.get(i + 1);
            txtArgs += (newArgs[i] + " ");
        }

        this.logger.info("Starting the application client with the arguments '" + txtArgs + "'.");

        this.logger.info("Starting client...");

        meth.invoke(null, new Object[] {newArgs});

        this.logger.debug("End of main method");

    }

    /**
     * Analyze arguments and extract parameters for the client container.
     * @throws Exception if there is an error when analyzing arguments
     */
    private void analyzeArgs() throws Exception {
        for (int argn = 0; argn < this.args.length; argn++) {
            String arg = this.args[argn];

            try {
                if (arg.equals("-tmpDir")) {
                    this.tmpDir = this.args[++argn];

                    continue;
                }

                if (arg.equals("-jarClient")) {
                    this.jarClient = this.args[++argn];

                    continue;
                }

                if (arg.equals("-cp")) {
                    this.classpath = this.args[++argn];
                    continue;
                }

                if (arg.equals("--help") || arg.equals("-help") || arg.equals("-h") || arg.equals("-?")) {
                    usage();
                    System.exit(1);
                }

                // Add argument to the application arguments
                this.appArgs.add(arg);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                // The next argument is not in the array
                throw new ClientContainerException("A required parameter was missing after the argument" + arg);
            }
        }
    }

    /**
     * Print the usage of this client.
     */
    private void usage() {
        System.out.println("Usage of this client :");
        System.out.println("-------------------------------------------------------------------");
        System.out.println("java -jar client.jar <client.jar|app.ear|className> [options]");
        System.out.println("-------------------------------------------------------------------");
        System.out.println(" -jarClient   : Specify the client jar to use of the ear if many.");
        System.out.println(" -tmpDir      : Specify the temp directory where unpack the ear.");
        System.out.println(" -cp          : Specify the classpath to use for the jar client.");
        System.out.println("-------------------------------------------------------------------");
        System.out.println("  --help  : Display this help.");
        System.out.println("  -help   : Display this help.");
        System.out.println("  -h      : Display this help.");
        System.out.println("  -?      : Display this help.");
        System.out.println("-------------------------------------------------------------------");
    }

    /**
     * Extract the client of an ear and analyze ear too.
     * @param earFile ear to be analyzed
     * @return the file of the client which was extracted
     * @throws Exception if the analyze and/or extract fails
     */
    private File extractAndAnalyzeEar(final File earFile) throws Exception {

        URL earUrl = null;

        try {
            earUrl = fileToURL2(earFile);
        } catch (URLUtilsException uue) {
            throw new ClientContainerException("Can not build an url with the filename '" + earFile + "'.", uue);
        }

        // Create classLoader
        URL[] arrURL = new URL[1];
        arrURL[0] = earUrl;

        // Temporary directory
        String tempDir = null;

        if (this.tmpDir != null) {
            // use specific directory
            tempDir = this.tmpDir;
            this.logger.debug("Use your specified temp directory '" + tempDir + "'.");
        } else {
            // use default
            tempDir = System.getProperty("java.io.tmpdir");
        }

        this.logger.debug("Using temp directory {0}", tempDir);

        // Can we write to ?
        File tmpFileDir = new File(tempDir);

        if (!tmpFileDir.exists() || !tmpFileDir.isDirectory()) {
            throw new ClientContainerException("The temp directory '" + tempDir + "' doesn't exist or is not a directory.");
        }

        if (!tmpFileDir.canWrite()) {
            throw new ClientContainerException("Can not write to the temporary directory '" + tempDir + "'.");
        }

        this.logger.info("Ear file = {0}", earFile);
        // Unpack the ear file and get the unpacked dir
        JarFile earJarFile = new JarFile(earFile);
        File rootFolder = new File(System.getProperty("java.io.tmpdir") + File.separator + System.getProperty("user.name")
                + File.separator + DEFAULT_FOLDER);
        rootFolder.mkdirs();

        File tmpFolder = new File(rootFolder, "TMP");
        tmpFolder.mkdirs();
        File file = new File(tmpFolder, earFile.getName() + ".new");

        try {
            FileUtils.unpack(earJarFile, file);
        } catch (FileUtilsException e) {
            throw new ClientContainerException("Cannot unpack '" + earJarFile + "'.", e);
        }

        File fClient = null;

        JNDIResolverHelper jndiResolverHelper = null;

        File clientFile = null;
        File[] files = file.listFiles();
        if (files != null) {
            if (this.jarClient != null) {
                int f = 0;
                File ff = null;
                boolean found = false;

                while (f < files.length && !found) {
                    ff = files[f];

                    if (ff.getPath().endsWith(this.jarClient)) {
                        found = true;
                        clientFile = ff;
                        this.logger.info("Found a matching client with the name {0}", ff);
                    }

                    f++;
                }

                if (!found) {
                    throw new ClientContainerException("No client with the name '" + this.jarClient
                            + "' was found in this Ear file");
                }
            }

            for (File f : files) {
                if (f.getName().toLowerCase().endsWith("_client.jar") && clientFile == null) {
                    clientFile = f;
                } else if (f.getName().toLowerCase().endsWith(".ear")) {
                    // ear, ignore
                    continue;
                } else {
                    IArchive archive = ArchiveManager.getInstance().getArchive(f);
                    if (jndiResolverHelper == null) {
                        this.containerJNDIResolver = new ContainerJNDIResolver(archive);
                        jndiResolverHelper = new JNDIResolverHelper(this.containerJNDIResolver);
                    }
                    Deployment dep = new Deployment(archive);
                    dep.analyze();
                    jndiResolverHelper.addDeployment(dep);
                }
            }
        }

        this.logger.debug("Resolver =" + this.containerJNDIResolver);

        if (clientFile == null) {
            throw new IllegalStateException("No client found");
        }

        // Take found or first found
        fClient = clientFile;

        this.logger.info("Use the application client '" + fClient + "' of the Ear '" + earUrl + "'.");

        return fClient;
    }

    /**
     * Gets the URL of user classpath (can be empty).
     * @return URL of user classpath (-cp arg)
     */
    private URL[] getUserClasspathUrls() {
        if (this.classpath == null) {
            return new URL[0];
        }
        String sep = File.pathSeparator;
        List<URL> clUser = new ArrayList<URL>();
        StringTokenizer tokenizer = new StringTokenizer(this.classpath, sep);
        while (tokenizer.hasMoreTokens()) {
            File file = new File(tokenizer.nextToken());
            try {
                clUser.add(fileToURL2(file));
            } catch (URLUtilsException uue) {
                this.logger.warn("Cannot transform to URL the file : '" + file + "'", uue);
            }
        }
        return clUser.toArray(new URL[0]);
    }

    /**
     * Build ENC environment used by this client.
     * @throws ClientContainerException if the ENC environment is not built.
     */
    private void buildENC() throws ClientContainerException {

        // Get NamingManager reference
        NamingManager namingManager = null;
        try {
            namingManager = NamingManager.getInstance();
        } catch (NamingException e) {
            throw new ClientContainerException("Cannot get a reference on the naming manager.", e);
        }

        Context javaCtx = null;
        // create env
       try {
            javaCtx = namingManager.createEnvironmentContext("client");
        } catch (NamingException e) {
            throw new ClientContainerException("Cannot build an environment context", e);
       }

        Context envCtx;
        try {
            envCtx = javaCtx.createSubcontext("comp/env");
        } catch (NamingException e) {
            throw new ClientContainerException("Cannot create subcontext comp/env", e);
        }

        // register environment for this client.
        namingManager.setClientContainerComponentContext(javaCtx);

        // add ejb-ref
        if (this.applicationClient != null) {
            for (EJBRef ejbRef : this.applicationClient.getEJBRefs()) {
                //get name
                String ejbRefName = ejbRef.getEjbRefName();

                // ejb-link
                String ejbLink = ejbRef.getEjbLink();

                // interface
                String home = ejbRef.getHome();
                String remote = ejbRef.getRemote();
                String itfName = null;
                if (home != null) {
                    itfName = home;
                } else {
                    itfName = remote;
                }

                // Resolve reference
                String jndiName;
                try {
                    jndiName = this.containerJNDIResolver.getUniqueName(itfName, ejbLink);
                } catch (EZBJNDIResolverException e) {
                    throw new ClientContainerException("Cannot get JNDI name for reference '" + ejbRefName + "'", e);
                }

                // bind reference
                try {
                    this.logger.debug("Binding ejb-ref-name {0} with jndi-name {1}", ejbRefName, jndiName);
                    envCtx.bind(ejbRefName, new LinkRef(jndiName));
                } catch (NamingException e) {
                    throw new ClientContainerException("Cannot bind name '" + ejbRefName + "' in java:comp/env", e);
                }
            }
        }
    }

}
