/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Deployment.java 2882 2008-04-17 17:02:43Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.ow2.easybeans.api.EZBContainerConfig;
import org.ow2.easybeans.deployment.annotations.analyzer.configurator.metadata.ejbjar.EjbJarArchiveMetadataConfigurator;
import org.ow2.easybeans.deployment.annotations.exceptions.ResolverException;
import org.ow2.easybeans.deployment.annotations.helper.ExtraMetadataHelper;
import org.ow2.easybeans.deployment.annotations.helper.ResolverHelper;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarArchiveMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarClassMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.LibrariesAnnotationMetadata;
import org.ow2.easybeans.deployment.xml.EJB3DeploymentDesc;
import org.ow2.easybeans.deployment.xml.EJB3DeploymentDescException;
import org.ow2.easybeans.deployment.xml.EasyBeansDeploymentDesc;
import org.ow2.easybeans.deployment.xml.helper.ExtractUsefullClasses;
import org.ow2.easybeans.deployment.xml.helper.MetadataMerge;
import org.ow2.easybeans.deployment.xml.helper.MetadataSpecificMerge;
import org.ow2.easybeans.deployment.xml.specific.EasyBeansDD;
import org.ow2.easybeans.deployment.xml.struct.EJB3;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.ArchiveScannerFactory;
import org.ow2.util.scan.api.ScanException;

/**
 * This class will parse given ejb-jar file and completes metadata by using
 * resolver.
 * @author Florent Benoit
 *         Contributors:
 *             S. Ali Tokmen (added support for prefixes)
 */
public class Deployment {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(Deployment.class);

    /**
     * Archive which will be analyzed.
     */
    private IArchive archive = null;

    /**
     * Extra archives.
     */
    private List<IArchive> extraArchives = null;

    /**
     * Configurator.
     */
    private EjbJarArchiveMetadataConfigurator ejbJarArchiveMetadataConfigurator;

    /**
     * Metadata of configurator.
     */
    private EjbJarArchiveMetadata ejbJarArchiveMetadata;

    /**
     * EasyBeans deployment configuration.
     */
    private EZBContainerConfig configuration;

    /**
     * Constructor.
     * @param archive Archive to deploy using the default deployment
     *                configuration.
     */
    public Deployment(final IArchive archive) {
        this.archive = archive;
        reset();
    }

    /**
     * Constructor.
     * @param configuration Deployment configuration.
     */
    public Deployment(final EZBContainerConfig configuration) {
        this(configuration.getArchive());
        this.configuration = configuration;
    }

    /**
     * Build a new Annotation Deployment Analyzer.
     */
    public void reset() {
        this.ejbJarArchiveMetadataConfigurator = new EjbJarArchiveMetadataConfigurator();
        this.ejbJarArchiveMetadata = this.ejbJarArchiveMetadataConfigurator.getEjbJarArchiveMetadata();
    }

    /**
     * Analyzes the jarFile.
     * @throws ScanException if analyze of jar file fails.
     * @throws ResolverException if resolver fails.
     * @throws EJB3DeploymentDescException if parsing fails.
     */
    @SuppressWarnings("boxing")
    public void analyze() throws ScanException, EJB3DeploymentDescException, ResolverException {

        ArchiveScanner archiveScanner = ArchiveScannerFactory.getInstance().createArchiveScanner(this.archive);
        archiveScanner.addArchiveConfigurator(this.ejbJarArchiveMetadataConfigurator);

        // Get EJB3 DD.
        EJB3 ejb3 = EJB3DeploymentDesc.getEjb3(this.archive);

        // Get Specific DD
        EasyBeansDD easyBeansDD = EasyBeansDeploymentDesc.getEasyBeansDD(this.archive);

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();

        // if metadata is complete then filter class to scan
        if (ejb3 != null && ejb3.isMetadataComplete()) {
            // disable annotation parsing
            this.ejbJarArchiveMetadataConfigurator.setAnnotationParsingDesactived(true);

            // get class to scan from deployment descriptor
            List<String> nextClassNameToScan = ExtractUsefullClasses.extract(ejb3);

            // first scan
            List<EjbJarClassMetadata> metadataWithDirectDependencies = new ArrayList<EjbJarClassMetadata>();
            Collection<EjbJarClassMetadata> ejbJarClassMetadataCollection = this.ejbJarArchiveMetadata
                    .getEjbJarClassMetadataCollection();
            archiveScanner.scanCollection(nextClassNameToScan);

            // first iteration : beans + interface + bean super class
            // after : interface + bean super class
            while (nextClassNameToScan.size() != 0) {
                nextClassNameToScan.clear();
                // try to add new class from new metadata
                for (EjbJarClassMetadata ejbJarClassMetadata : ejbJarClassMetadataCollection) {
                    if (metadataWithDirectDependencies.contains(ejbJarClassMetadata)) {
                        continue;
                    }
                    // add direct dependencies
                    nextClassNameToScan.add(ejbJarClassMetadata.getSuperName());
                    nextClassNameToScan.addAll(Arrays.asList(ejbJarClassMetadata.getInterfaces()));
                    // specify to ignore this metadata in future
                    metadataWithDirectDependencies.add(ejbJarClassMetadata);
                }
                // continue the scan
                archiveScanner.scanCollection(nextClassNameToScan);
            }

            // scan finished
            archiveScanner.endScan();
        } else {
            // enable annotation parsing because metadata descriptor is not complete
            this.ejbJarArchiveMetadataConfigurator.setAnnotationParsingDesactived(false);
            archiveScanner.scanAll();
        }

        // time if debugging
        if (logger.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            if (logger.isDebugEnabled()) {
                logger.debug("Analyze of file {0} took {1} ms.", this.archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
            }
        }

        LibrariesAnnotationMetadata librariesAnnotationMetadata = new LibrariesAnnotationMetadata();
        // Extra archives ?
        if (this.extraArchives != null) {
            List<EjbJarArchiveMetadata> ejbJarAnnotationMetadataList = new ArrayList<EjbJarArchiveMetadata>();

            // Analyze the extra archives (with annotation parsing and blind
            // scan)
            for (IArchive extraArchive : this.extraArchives) {
                ArchiveScanner extraArchiveScanner = ArchiveScannerFactory.getInstance().createArchiveScanner(extraArchive);
                EjbJarArchiveMetadataConfigurator ejbJarExtraArchiveMetadataConfigurator =
                    new EjbJarArchiveMetadataConfigurator();
                extraArchiveScanner.addArchiveConfigurator(ejbJarExtraArchiveMetadataConfigurator);
                extraArchiveScanner.scanAll();
                ejbJarAnnotationMetadataList.add(ejbJarExtraArchiveMetadataConfigurator.getEjbJarArchiveMetadata());
            }
            librariesAnnotationMetadata.setEjbJarAnnotationMetadataList(ejbJarAnnotationMetadataList);
        }

        // Extra metadata ? complete them
        ExtraMetadataHelper.complete(this.ejbJarArchiveMetadata, librariesAnnotationMetadata);

        // prepare the metadata for merge
        this.ejbJarArchiveMetadata.setEjb3(ejb3);

        // Merge DD with annotations metadata
        MetadataMerge.merge(this.ejbJarArchiveMetadata);

        // Merge specific DD
        this.ejbJarArchiveMetadata.setEasyBeansDD(easyBeansDD);
        MetadataSpecificMerge.merge(this.ejbJarArchiveMetadata);

        // Complete metadata
        long tResolverStart = System.currentTimeMillis();
        ResolverHelper.resolve(this.ejbJarArchiveMetadata);
        // time if debugging
        if (logger.isDebugEnabled()) {
            long tResolverEnd = System.currentTimeMillis();
            if (logger.isDebugEnabled()) {
                logger.debug("Resolver on metadata from {0} took {1} ms.'", this.archive.getName(),
                        (tResolverEnd - tResolverStart));
            }
        }

        if (logger.isDebugEnabled()) {
            for (EjbJarClassMetadata classAnnotationMetadata : this.ejbJarArchiveMetadata.getEjbJarClassMetadataCollection()) {
                logger.debug("Result for class = " + classAnnotationMetadata);
            }
        }
    }

    /**
     * @return the archive metadata
     */
    public EjbJarArchiveMetadata getEjbJarArchiveMetadata() {
        return this.ejbJarArchiveMetadata;
    }

    /**
     * Add extra archives for finding classes.
     * @param extraArchives the given archives.
     */
    public void setExtraArchives(final List<IArchive> extraArchives) {
        this.extraArchives = extraArchives;
    }

    /**
     * @return the archive of this deployment.
     */
    public IArchive getArchive() {
        return this.archive;
    }

    /**
     * @return the EasyBeans deployment configuration, will be null if the
     *         {@link Deployment#Deployment(IArchive)} constructor has been
     *         used.
     */
    public EZBContainerConfig getConfiguration() {
        return this.configuration;
    }
}
