/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbEJBVisitor.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.annotations.analyzer.visitors;

import org.ow2.easybeans.asm.Type;

import org.ow2.easybeans.deployment.annotations.analyzer.AnnotationType;
import org.ow2.easybeans.deployment.annotations.impl.JEjbEJB;
import org.ow2.easybeans.deployment.metadata.interfaces.IEjbEJB;

/**
 * This class manages the handling of &#64;{@link javax.ejb.EJB} annotation.
 * @param <T> An implementation of IAnnotationEJB interface.
 * @author Florent Benoit
 */
public class JavaxEjbEJBVisitor<T extends IEjbEJB>
          extends AbsAnnotationVisitor<T> implements AnnotationType {

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * Bean interface attribute of the annotation.
     */
    private static final String BEAN_INTERFACE = "beanInterface";

    /**
     * Bean name attribute of the annotation.
     */
    private static final String BEAN_NAME = "beanName";

    /**
     * Mapped name attribute of the annotation.
     */
    private static final String MAPPED_NAME = "mappedName";

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/EJB;";

    /**
     * Internal object used representing &#64;{@link javax.ejb.EJB} annotation.
     */
    private JEjbEJB jEjbEJB = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method metadata
     */
    public JavaxEjbEJBVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        jEjbEJB = new JEjbEJB();
    }


    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.easybeans.asm.Type}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(NAME)) {
            jEjbEJB.setName((String) value);
        } else if (name.equals(BEAN_INTERFACE)) {
            Type type = (Type) value;
            jEjbEJB.setBeanInterface(type.getClassName());
        } else if (name.equals(BEAN_NAME)) {
            jEjbEJB.setBeanName((String) value);
        } else if (name.equals(MAPPED_NAME)) {
            jEjbEJB.setMappedName((String) value);
        }
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object
        getAnnotationMetadata().setJEjbEJB(jEjbEJB);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }


    /**
     * @return Internal object used representing &#64;{@link javax.ejb.EJB} annotation.
     */
    protected JEjbEJB getJEjbEJB() {
        return jEjbEJB;
    }

    /**
     * Sets the jEjbEJB object.
     * @param jEjbEJB the object which replaced the previous one.
     */
    protected void setJEjbEJB(final JEjbEJB jEjbEJB) {
        this.jEjbEJB = jEjbEJB;
    }

}
