/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestLoading.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.tests.deploymentdesc;

import static org.ow2.easybeans.util.url.URLUtils.fileToURL2;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

import java.io.File;
import java.net.URL;
import java.util.List;

import org.ow2.easybeans.deployment.xml.parsing.EJB3DeploymentDescLoader;
import org.ow2.easybeans.deployment.xml.parsing.ParsingException;
import org.ow2.easybeans.deployment.xml.struct.EJB3;
import org.ow2.easybeans.deployment.xml.struct.EnterpriseBeans;
import org.ow2.easybeans.deployment.xml.struct.Session;
import org.ow2.easybeans.deployment.xml.struct.common.EnvEntry;
import org.ow2.easybeans.util.url.URLUtilsException;
import org.ow2.easybeans.util.xml.DocumentParserException;
import org.testng.annotations.Test;
import org.xml.sax.SAXException;

/**
 * Tests the parsing of an ejb-jar.xml file.
 * @author Florent Benoit
 */
public class TestLoading {

    /**
     * Prefix for XML files.
     */
    private static final String PREFIX = TestLoading.class.getPackage().getName().replace(".", "/") + "/xmls/";

    /**
     * Prefix of src for tests.
     */
    private static final String TEST_DIR_PREFIX = "tests/src/main/java/";

    /**
     * Gets an url for a given ejb-jar.xml filename.
     * @param fileName the name of the fiel to search.
     * @return an URL if found
     * @throws ParsingException if no URL is found.
     */
    private static URL getURL(final String fileName) throws ParsingException {
        URL url = TestLoading.class.getResource(PREFIX + fileName);
        if (url == null) {
            // try with filesystem
            File file = new File(TEST_DIR_PREFIX + PREFIX + fileName);
            if (file.exists()) {
                try {
                    return fileToURL2(file);
                } catch (URLUtilsException e) {
                    throw new ParsingException("Cannot transform file into url", e);
                }
            }
        }
        if (url == null) {
            throw new ParsingException("URl for '" + fileName + "' was not found.");
        }
        return url;
    }

    /**
     * Parse a given filename and gets the resulting parsing.
     * @param fileName the name of the file to parse
     * @return a struct representing the EJB
     * @throws ParsingException if parsing has failed.
     */
    private EJB3 getEjb3(final String fileName) throws ParsingException {
        URL urlEjb01 = getURL(fileName);

        return EJB3DeploymentDescLoader.loadDeploymentDescriptor(urlEjb01);

    }

    /**
     * Checks that an xml file can be parsed by analyzing the result.
     * @throws ParsingException if xml is not parsed.
     */
    @Test
    public void analyzeParsedXml() throws ParsingException {
        EJB3 ejb3 = getEjb3("ejb-jar01.xml");

        // Ensure that there is a bean named TestBean.
        EnterpriseBeans enterpriseBeans = ejb3.getEnterpriseBeans();
        assertNotNull(enterpriseBeans);

        // there are session beans
        List<Session> sessionList = enterpriseBeans.getSessionList();
        assertNotNull(sessionList);
        assertTrue(sessionList.size() > 0);

        // first bean = TestBean
        Session testBean = sessionList.get(0);
        assertNotNull(testBean);
        assertEquals("TestBean", testBean.getEjbName());

        // Env-Entry
        List<EnvEntry> envEntries = testBean.getEnvEntryList();
        // there are envEntries
        assertNotNull(envEntries);
        assertTrue(envEntries.size() > 0);

        EnvEntry envEntry = envEntries.get(0);
        assertEquals("testBoolean", envEntry.getEnvEntryName());
        assertEquals("java.lang.Boolean", envEntry.getEnvEntryType());
        assertEquals("false", envEntry.getEnvEntryValue());

    }

    /**
     * Checks that an xml file is invalid.
     */
    @Test
    public void error01() {
        // don't use here @ExpectedExceptions
        try {
            getEjb3("ejb-jar-error01.xml");
            assert false;
        } catch (ParsingException e) {
            Throwable t = e.getCause();
            assertTrue(t instanceof DocumentParserException);
            DocumentParserException dpe = (DocumentParserException) t;
            Throwable tt = dpe.getCause();
            assertTrue(tt instanceof SAXException);
            SAXException se = (SAXException) tt;
            // Check error message
            assertTrue(se.getMessage().contains("The content of element 'session' is not complete"));
        }

    }

}
